# -*- coding: utf-8 -*-
#
# WaiverDB documentation build configuration file, created by
# sphinx-quickstart on Wed Mar 15 17:21:27 2017.
#
# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys

# This will cause the Flask application to be created with development configs
os.environ['DEV'] = 'true'

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '../')))
import waiverdb  # noqa: E402


# -- General configuration ------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
needs_sphinx = '1.3'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.doctest',
    'sphinx.ext.intersphinx',
    'sphinx.ext.napoleon',
    'sphinx.ext.viewcode',
    'sphinxcontrib.autohttp.flask',
    'sphinxcontrib.issuetracker',
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = ['.rst', '.md']
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = u'WaiverDB'
copyright = u'2017, Red Hat, Inc. and others'
author = u'Red Hat, Inc. and others'

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = '.'.join(waiverdb.__version__.split('.')[:2])
# The full version, including alpha/beta/rc tags.
release = waiverdb.__version__

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = None

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This patterns also effect to html_static_path and html_extra_path
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = False


# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'alabaster'

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#
# html_theme_options = {}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = []


# -- Options for HTMLHelp output ------------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = 'WaiverDBdoc'


# -- Options for LaTeX output ---------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #
    # 'papersize': 'letterpaper',

    # The font size ('10pt', '11pt' or '12pt').
    #
    # 'pointsize': '10pt',

    # Additional stuff for the LaTeX preamble.
    #
    # 'preamble': '',

    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    (master_doc, 'WaiverDB.tex', u'WaiverDB Documentation',
     u'Red Hat, Inc. and others', 'manual'),
]


# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (master_doc, 'waiverdb', u'WaiverDB Documentation',
     [author], 1)
]


# -- Options for Texinfo output -------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (master_doc, 'WaiverDB', u'WaiverDB Documentation',
     author, 'WaiverDB', 'One line description of project.',
     'Miscellaneous'),
]


# -- Options for Epub output ----------------------------------------------

# Bibliographic Dublin Core info.
epub_title = project
epub_author = author
epub_publisher = author
epub_copyright = copyright

# The unique identifier of the text. This can be a ISBN number
# or the project homepage.
#
# epub_identifier = ''

# A unique identification for the text.
#
# epub_uid = ''

# A list of files that should not be packed into the epub file.
epub_exclude_files = ['search.html']


# Example configuration for intersphinx: refer to the Python standard library.
intersphinx_mapping = {'https://docs.python.org/2/': 'python-intersphinx.inv'}

# Configuration for issue tracker magic linking.
issuetracker = None
issuetracker_url = 'https://pagure.io/'
issuetracker_project = 'waiverdb'

# Until Pagure support is merged: https://github.com/ignatenkobrain/sphinxcontrib-issuetracker/pull/15
import requests
from sphinxcontrib.issuetracker import Issue
PAGURE_URL = '{0.url}/{0.project}/issue/{1}'
PAGURE_API_URL = '{0.url}/api/0/{0.project}/issue/{1}'
HEADERS = {
    'User-Agent': 'sphinxcontrib-issuetracker'
}
def get(app, url):
    try:
        response = requests.get(url, headers=HEADERS)
        if response.status_code == requests.codes.ok:
            return response
        elif response.status_code != requests.codes.not_found:
            msg = 'GET {0.url} failed with code {0.status_code}'
            app.warn(msg.format(response))
    except IOError as e:
        msg = 'GET {0} failed with error: {1}'
        app.warn(msg.format(url, e))
def lookup_pagure_issue(app, tracker_config, issue_id):
    if not tracker_config.url:
        raise ValueError('URL required, try: https://pagure.io/')
    issue_url = PAGURE_URL.format(tracker_config, issue_id)
    response = get(app, PAGURE_API_URL.format(tracker_config, issue_id))
    if response:
        title = response.json()['title']
        closed = response.json()['status'] != 'Open'
        return Issue(id=issue_id, title=title, closed=closed, url=issue_url)
    else:
        return Issue(id=issue_id, title=None, closed=False, url=issue_url)

def setup(app):
    app.connect('issuetracker-lookup-issue', lookup_pagure_issue)
