"""Initializes migration

Revision ID: 0a27a8ad723a
Revises: None
Create Date: 2017-11-21 10:57:17.056587

"""

# revision identifiers, used by Alembic.
revision = '0a27a8ad723a'
down_revision = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('waiver',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('result_id', sa.Integer(), nullable=False),
    sa.Column('username', sa.String(length=255), nullable=False),
    sa.Column('product_version', sa.String(length=200), nullable=False),
    sa.Column('waived', sa.Boolean(), nullable=False),
    sa.Column('comment', sa.Text(), nullable=True),
    sa.Column('timestamp', sa.DateTime(), nullable=True),
    sa.PrimaryKeyConstraint('id')
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('waiver')
    # ### end Alembic commands ###
